/*
 * ----------------------------------------------------------------------------
 * "THE BEER-WARE LICENSE" (Revision 42):
 * <alucassch@gmail.com> and <brunomarcosespindola@gmail.com> wrote this file.
 * As long as you retain this notice you can do whatever you want with this stuff.
 * If we meet some day, and you think this stuff is worth it, you can buy
 * me a beer in return André Lucas Schlichting e Bruno Marcos Espindola
 * ----------------------------------------------------------------------------
 */

/*
 * Tradução da Licensa
 *  ----------------------------------------------------------------------------
 * "THE BEER-WARE LICENSE" (Revision 42):
 * <alucassch@gmail.com> e <brunomarcosespindola@gmail.com>  escreveram esse código.
 * Enquanto você possui-lo, note que  você pode fazer o que desejar com ele. Se um
 * dia a gente se conhecer, e você achar que isso funciona, você pode nos comprar
 * uma cerveja em homenagem a André Lucas Schlichting e Bruno Marcos Espindola
 * ----------------------------------------------------------------------------
 */

package Protocolo;

import Damas.Pecas;
import java.io.Serializable;
import java.util.ArrayList;

/**
 *Mensagens que serão utilizadas para a troca de informações entre os clientes e o servidor
 * Cada construtor recebe parâmetros diferentes, o que caracteriza um tipo de mensagem diferente, carregando diferentes tipos de dados
 * Dependendo do caso, deve-se verificar os dados que serão enviados e usar o construtor correto
 *
 */
public class Mensagem implements Serializable {

    public int codigo; //armazena o codigo da mensagem
    public String texto; //armazena mensagens de texto, para usar no batepapo ou mensagens onde apenas o código é importante, nesse caso, manda-se uma string vazia
    public int SelecionadoX; //armazena a coordenada X da peça selecionada
    public int SelecionadoY; //armazena a coordenada Y da peça selecionada
    public int destinoX; //armazena a coordenada X do destino da peça
    public int destinoY; //armazena a coordenada Y do destino da peça
    public ArrayList<String> maiorCaminho; //armazena o maior caminho disponível
    public ArrayList<String> pecasComer; //armazena as coordenadas das peças que podem ser comidas
    public boolean primeiro; //variável que envia aos clientes quem será o jogador amarelo e quems erá o jogador vermelho
    public Pecas[][] pecas; //variáve que serve para enviar o tabuleiro atualizado aos clientes
    public boolean vez; //variável que muda a vez dos jogadores
    public boolean corJogador; //variável que indica qual a cor do jogador que executou a jogada
    public int contadorAmarelas; //variável que envia a quantidade de peças amarelas
    public int contadorVermelhas; //variável que envia a quantidade de peças vermelhas

    public Mensagem(int cod, String t) {
        this.codigo = cod;
        this.texto = t;
        this.SelecionadoX = -1;
        this.SelecionadoY = -1;
        this.destinoX = -1;
        this.destinoY = -1;
        this.maiorCaminho = null;
        this.pecasComer = null;
        this.primeiro = false;
        this.vez = false;
        this.corJogador = false;
        this.contadorAmarelas = 0;
        this.contadorVermelhas = 0;
    }

    public Mensagem(int cod, int contAma, int contVerm) {
        this.codigo = cod;
        this.contadorAmarelas = contAma;
        this.contadorVermelhas = contVerm;
        this.texto = "";
        this.SelecionadoX = -1;
        this.SelecionadoY = -1;
        this.destinoX = -1;
        this.destinoY = -1;
        this.maiorCaminho = null;
        this.pecasComer = null;
        this.primeiro = false;
        this.vez = false;
        this.corJogador = false;
    }



    public Mensagem(int cod, Pecas[][] pec) {
        this.codigo = cod;
        this.texto = "";
        this.SelecionadoX = -1;
        this.SelecionadoY = -1;
        this.destinoX = -1;
        this.destinoY = -1;
        this.maiorCaminho = null;
        this.pecasComer = null;
        this.primeiro = false;
        this.pecas = pec;
        this.vez = false;
        this.corJogador = false;
        this.contadorAmarelas = 0;
        this.contadorVermelhas = 0;
    }

    public Mensagem(int cod, Pecas[][] pec, boolean b) {
        this.codigo = cod;
        this.texto = "";
        this.SelecionadoX = -1;
        this.SelecionadoY = -1;
        this.destinoX = -1;
        this.destinoY = -1;
        this.maiorCaminho = null;
        this.pecasComer = null;
        this.primeiro = false;
        this.pecas = pec;
        this.vez = b;
        this.corJogador = false;
        this.contadorAmarelas = 0;
        this.contadorVermelhas = 0;
    }

    public Mensagem(int cod, String t, boolean p) {
        this.codigo = cod;
        this.texto = t;
        this.SelecionadoX = -1;
        this.SelecionadoY = -1;
        this.destinoX = -1;
        this.destinoY = -1;
        this.maiorCaminho = null;
        this.pecasComer = null;
        this.primeiro = p;
        this.vez = false;
        this.corJogador = false;
        this.contadorAmarelas = 0;
        this.contadorVermelhas = 0;
    }

    public Mensagem(int cod, int ox, int oy, int dx, int dy, boolean b) {
        this.codigo = cod;
        this.texto = "";
        this.SelecionadoX = ox;
        this.SelecionadoY = oy;
        this.destinoX = dx;
        this.destinoY = dy;
        this.maiorCaminho = null;
        this.pecasComer = null;
        this.primeiro = false;
        this.vez = false;
        this.corJogador = b;
        this.contadorAmarelas = 0;
        this.contadorVermelhas = 0;
    }

    public Mensagem(int cod, ArrayList<String> caminho) {
        this.codigo = cod;
        this.texto = "";
        this.SelecionadoX = -1;
        this.SelecionadoY = -1;
        this.destinoX = -1;
        this.destinoY = -1;
        this.maiorCaminho = caminho;
        this.pecasComer = null;
        this.primeiro = false;
        this.vez = false;
        this.corJogador = false;
        this.contadorAmarelas = 0;
        this.contadorVermelhas = 0;
    }

    public Mensagem(int cod, boolean minhaCor) {
        this.codigo = cod;
        this.texto = "";
        this.SelecionadoX = -1;
        this.SelecionadoY = -1;
        this.destinoX = -1;
        this.destinoY = -1;
        this.maiorCaminho = null;
        this.pecasComer = null;
        this.primeiro = minhaCor;
        this.vez = false;
        this.corJogador = false;
        this.contadorAmarelas = 0;
        this.contadorVermelhas = 0;
    }
}
