/*
 * ----------------------------------------------------------------------------
 * "THE BEER-WARE LICENSE" (Revision 42):
 * <alucassch@gmail.com> and <brunomarcosespindola@gmail.com> wrote this file.
 * As long as you retain this notice you can do whatever you want with this stuff.
 * If we meet some day, and you think this stuff is worth it, you can buy
 * me a beer in return André Lucas Schlichting e Bruno Marcos Espindola
 * ----------------------------------------------------------------------------
 */

/*
 * Tradução da Licensa
 *  ----------------------------------------------------------------------------
 * "THE BEER-WARE LICENSE" (Revision 42):
 * <alucassch@gmail.com> e <brunomarcosespindola@gmail.com>  escreveram esse código.
 * Enquanto você possui-lo, note que  você pode fazer o que desejar com ele. Se um
 * dia a gente se conhecer, e você achar que isso funciona, você pode nos comprar
 * uma cerveja em homenagem a André Lucas Schlichting e Bruno Marcos Espindola
 * ----------------------------------------------------------------------------
 */

package Servidor;

import Damas.Pecas;
import java.io.EOFException;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.net.Socket;
import java.util.ArrayList;
import Protocolo.Mensagem;

public class ThreadServidor extends Thread {

    private Socket conexao; //armazena o socket da conexão
    private ArrayList<ObjectOutputStream> clientes; //arraylist que contém os fluxos de saída dos clientes
    private ArrayList<String> nClientes; //arraylist que contem a quantidade de clientes
    private Pecas[][] tabuleiro; //objeto da classe Peças
    private int indice; //armazena a posição do arraylist que foram comidas as peças
    private boolean jogadorVez; //armazena o jogador que está fazendo a jogada
    private ArrayList<String> caminhos; //armazena os caminhos que podem ser feitos (variável auxiliar)
    private ArrayList<String> pecasComer; //armazena as peças que podem ser comidas (variável auxiliar)
    private ArrayList<String> pecasParaComer; //armazena os maiores vetores de peças que podem ser comidas
    private ArrayList<String> maiorCaminho; //armazeba os maiores vetores de caminhos possiveis para comer
    private ArrayList<String> al; //variável auxiliar para executar o maior caminho
    private int contadorVermelhas; //armazena a quantidade de peças vermelhas restantes
    private int contadorAmarelas; //armazena a quantidade de peças amarelas

    public ThreadServidor(Socket c, ArrayList<ObjectOutputStream> cli, ArrayList<String> nC, Pecas[][] tabuleiro, boolean vez) {
        this.conexao = c;
        this.clientes = cli;
        this.nClientes = nC;
        this.jogadorVez = vez;
        this.tabuleiro = tabuleiro;
        this.caminhos = new ArrayList<String>();
        this.pecasComer = new ArrayList<String>();
        this.pecasParaComer = new ArrayList<String>();
        this.maiorCaminho = new ArrayList<String>();
        this.al = new ArrayList<String>();
        this.contadorAmarelas = 0;
        this.contadorVermelhas = 0;

    }

    @Override
    public void run() {
        ObjectInputStream fluxoEntrada = null;
        ObjectOutputStream fluxoSaida = null;
        String nomeDoCliente = "";
        try {
            // Objetos serão transportados pelo sockets
            fluxoEntrada = new ObjectInputStream(conexao.getInputStream());
            fluxoSaida = new ObjectOutputStream(conexao.getOutputStream());

            Mensagem recebida = null;

            //adicionado o fluxo de saida do cliente no arraylist
            //isso permite ao servidor enviar mensagens para todos os clientes conectados
            clientes.add(fluxoSaida);

            //esperando o cliente enviar seu nome.
            //mensagem com código 15
            recebida = (Mensagem) fluxoEntrada.readObject();

            // se a conexao do cliente cair, encerre a thread
            if (recebida == null) {
                return;
            }

            Mensagem enviada = null;

            if (recebida.codigo == 15) {
                nomeDoCliente = recebida.texto;
                System.out.println(nomeDoCliente + " conectou...");

                nClientes.add(nomeDoCliente);



                if (clientes.size() == 2) {

                    for (ObjectOutputStream saidaCli : clientes) {
                        // evitando enviar a mensagem para o cliente que a originou
                        if (saidaCli != fluxoSaida) {
                            System.out.println("enviando cliente 1");
                            saidaCli.writeObject(new Mensagem(15, nClientes.get(1), true));
                        } else {
                            System.out.println("enviando cliente 2");
                            saidaCli.writeObject(new Mensagem(15, nClientes.get(0), false));
                        }
                        saidaCli.flush();
                        saidaCli.reset();

                    }
                }
            }

            do {
                recebida = (Mensagem) fluxoEntrada.readObject(); //espera mensagens do cliente

                //codigo 13 indica um movimento ou peças que foram comidas
                if (recebida.codigo == 13) {
                    this.jogadorVez = recebida.corJogador; //salva em jogadorVez o cliente que fez a jogada
                    int i = this.mover(recebida.SelecionadoX, recebida.SelecionadoY, recebida.destinoX, recebida.destinoY); //chama o método mover para verificar a viabilidade da jogada
                    if (i == 0) { // i = 0 indica um movimento simples da peça
                        this.tabuleiro[recebida.destinoX][recebida.destinoY] = this.tabuleiro[recebida.SelecionadoX][recebida.SelecionadoY]; //'move' a peça de lugar, para o destino escolhido
                        this.tabuleiro[recebida.SelecionadoX][recebida.SelecionadoY] = null; //apaga a peça da origem dela, pois ela ja foi movida. Na verdade ela foi copiada da origem para o destino, então é necessário apaga-la
                        this.verificaDama();//chama o método verificaDama para verificar se com a jogada foi criada alguma dama

                        //todas as verificações foram executadas e o servidor manda uma resposta para os clientes
                        for (ObjectOutputStream saidaCli : clientes) {
                            //mensagem com o código 20 indica que foi executada uma jogada por um cliente, está sendo enviado também o novo tabuleiro já com as peças alteradas e está sendo trocada a vez do jogador
                            saidaCli.writeObject(new Mensagem(20, tabuleiro, !this.jogadorVez));
                            saidaCli.writeObject(new Mensagem(33, recebida.SelecionadoX,recebida.SelecionadoY,recebida.destinoX,recebida.destinoY,jogadorVez));
                            saidaCli.flush();
                            saidaCli.reset();
                        }

                    }

                    if (i == 1) { //i =  1 indica que um movimento onde foi comida alguma ou algumas peças encontradas durante a recursividade
                        //move a peça e apaga ela da origem
                        this.tabuleiro[recebida.destinoX][recebida.destinoY] = this.tabuleiro[recebida.SelecionadoX][recebida.SelecionadoY];
                        this.tabuleiro[recebida.SelecionadoX][recebida.SelecionadoY] = null;
                        //salva em uma string, as posições das peças que foram comidas através da variável 'indice' que foi salva durante o método 'mover'
                        String come = pecasParaComer.get(indice);
                        /*if (jogadorVez) {
                        this.pecasComidas = (come.length() / 2);
                        } else {

                        this.pecasComidas = (come.length() / 2);
                        }*/
                        //apaga as peças da String anterior
                        for (int p = 0; p < come.length(); p = p + 2) {
                            this.tabuleiro[Integer.parseInt(come.substring(p, p + 1))][Integer.parseInt(come.substring(p + 1, p + 2))] = null;
                        }
                        this.verificaDama(); //chama o método verificaDama para verificar se com a jogada foi criada alguma dama
                        //todas as verificações foram executadas e o servidor manda uma resposta para os clientes
                        for (ObjectOutputStream saidaCli : clientes) {
                            //mensagem com o código 20 indica que foi executada uma jogada por um cliente, está sendo enviado também o novo tabuleiro já com as peças alteradas e está sendo trocada a vez do jogador
                            saidaCli.writeObject(new Mensagem(20, tabuleiro, !this.jogadorVez));
                            saidaCli.writeObject(new Mensagem(33, recebida.SelecionadoX,recebida.SelecionadoY,recebida.destinoX,recebida.destinoY,jogadorVez));
                            saidaCli.flush();
                            saidaCli.reset();
                        }
                    }

                    if (i == 2) { //i = 2 indica que alguma peça deveria ser comida e não foi, o que deve obrigar o jogador refazer a jogada
                        for (ObjectOutputStream saidaCli : clientes) {
                            //mensagem com o código 65 indica que que o jogador errou a jogada e deve refazê-la comendo as pelas indicadas
                            //está sendo enviado aos clientes o arraylist 'al' com as posições do maior caminho, para indicar na tela do cliente como deve ser executado o movimento
                            saidaCli.writeObject(new Mensagem(65, al));
                            saidaCli.flush();
                            saidaCli.reset();
                        }
                    }

                    //zera os dois arraylists para evitar problemas
                    al = new ArrayList<String>();
                    maiorCaminho = new ArrayList<String>();

                }
                //faz a verificação da quantidade de peças no jogo para indicar o fim do mesmo
                this.verificaQuantidade();
                int codi = 48; //continua jogo
                if (this.contadorAmarelas == 0) {
                    codi = 49; //vermelho ganhou
                } else if (this.contadorVermelhas == 0) {
                    codi = 50; //amarelo ganhou;
                } else if (this.contadorAmarelas == 1 && this.contadorVermelhas == 1) { //caso sobre apenas uma peça de cada jogador o jogo empata
                    codi = 51; //jogo empatou                    
                }

                //envia para os clientes a quantidade de peças o o código indicando quem ganhou, se empatou, ou se deve-se continuar o jogo, dependendo do valor da variável 'codi'
                for (ObjectOutputStream saidaCli : clientes) {
                    saidaCli.writeObject(new Mensagem(codi, contadorAmarelas, contadorVermelhas));
                    saidaCli.flush();
                    saidaCli.reset();
                }

                //codigo 100 indica uma mensagem de bate-papo
                if (recebida.codigo == 100) {
                    System.out.println(nomeDoCliente + ": " + recebida.texto);
                    for (ObjectOutputStream saidaCli : clientes) {
                        // evitando enviar a mensagem para o cliente que a originou
                        if (saidaCli != fluxoSaida) {
                            //envia para o outro cliente, a mensagem recebida
                            saidaCli.writeObject(recebida);
                            saidaCli.flush();
                            saidaCli.reset();
                        }
                    }
                }

                if (recebida.codigo == 24) {
                    for (ObjectOutputStream saidaCli : clientes) {
                        //saidaCli.writeObject(new Mensagem(20, tabuleiro, !this.jogadorVez));
                        if (saidaCli != fluxoSaida) {
                            saidaCli.writeObject(new Mensagem(25, ""));
                            saidaCli.flush();
                            saidaCli.reset();
                        }
                    }
                }

                if (recebida.codigo == 26) {
                    if (recebida.primeiro) {
                        //vai reiniciar
                        this.tabuleiroJogoNovo();
                        for (ObjectOutputStream saidaCli : clientes) {
                            //saidaCli.writeObject(new Mensagem(20, tabuleiro, this.jogadorVez));
                            saidaCli.writeObject(new Mensagem(27, this.tabuleiro, !this.jogadorVez));

                            saidaCli.flush();
                            saidaCli.reset();
                        }
                    }
                    if (recebida.primeiro == false) {
                        for (ObjectOutputStream saidaCli : clientes) {
                            if (saidaCli != fluxoSaida) {
                                saidaCli.writeObject(new Mensagem(28, ""));
                                saidaCli.flush();
                                saidaCli.reset();
                            }

                        }
                    }
                }

            } while (recebida.codigo != 10);
        } catch (EOFException eof) {
            System.out.println(nomeDoCliente + " saiu...");
        } catch (IOException ex) {
            System.err.println(ex.toString());
        } catch (Exception e) {
            System.err.println(e.toString());
        } finally {
            try {
                // removendo fluxo de saida do arraylist, afinal o cliente desconectou
                clientes.remove(fluxoSaida);

                // fechando fluxos
                fluxoEntrada.close();
                fluxoSaida.close();
                conexao.close();

            } catch (IOException ex) {
                System.err.println("finally: " + ex.toString());
            }
        }
    }

    public void tabuleiroJogoNovo() {
        for (int lin = 0; lin < 8; lin++) {
            for (int col = 0; col < 8; col++) {
                tabuleiro[lin][col] = null;
            }
        }

        for (int lin = 0; lin < 8; lin++) { //percorrendo linhas
            // determinando a cor da peca
            // false = preta
            // true = branca
            boolean cor = (lin > 4) ? true : false;
            for (int col = 0; col < 8; col++) { //percorrendo colunas
                if ((lin != 3 && lin != 4) && (((lin % 2 == 0) && (col % 2 == 1)) || ((lin % 2 == 1) && (col % 2 == 0)))) {
                    this.tabuleiro[lin][col] = new Pecas(cor, false);
                }
            }
        }

    }

    public void verificaQuantidade() {
        contadorAmarelas = 0;
        contadorVermelhas = 0;
        for (int i = 0; i <= 7; i++) {
            for (int j = 0; j <= 7; j++) {
                if (tabuleiro[i][j] != null) {
                    if (tabuleiro[i][j].getCor()) {
                        contadorAmarelas++;
                    } else {
                        contadorVermelhas++;
                    }
                }
            }
        }
    }

    /*
     * recebe como parâmetros as posições e origens das casas selecionadas e enviadas pelo cliente
     * verifica se a jogada pode ser feita
     */
    private int mover(int ox, int oy, int dx, int dy) {

        int resultado = -1;

        al = this.procurarTodosCaminhos(); //chama o método que verifica todo o tabuleiro a procura de peças que podem ser comidas (recursividade)

        //utiliza o ArrayList 'al' para verificar se as posições recebidas no método são equivalentes as posições possivelmente encontradas no método 'procurarTodosCaminhos'
        for (int i = 0; i < al.size(); i++) {
            String temp = al.get(i);
            String origem = al.get(i).substring(0, 2);//separa os dois primeiros caracteres do arraylist. São a posição da peça de origem
            int tamanho = temp.length();
            String destino = al.get(i).substring(tamanho - 2, tamanho);//serpara os dois últimos caracteres do arraylist. São a posiçao de destino da peça
            //salva em strings para poder comparar
            String casaSelecionada = ox + "" + oy;
            String casaDestino = dx + "" + dy;
            if (casaSelecionada.equals(origem) && casaDestino.equals(destino)) { //compara se a posição clicada e recebida no método, é a mesma encontrada pelo programa na recursividade
                this.indice = i; //salva qual posição do arraylist foi correspondente a utilizada
                return 1; //retorna 1 para indicar que foram encontradas peças para comer, e que o cliente clicou para comer essas peças
            }
        }

        //se chegar a essa linha e passar na verificação desse if, significa que haviam peças para serem comidas, mas o cliente clicou errado
        //retorna então 2, para 'obrigar' e sinalizar o erro ao cliente.
        if (al.size() > 0) {
            return 2;
        }

        //se chegar a essa linha, nenhuma das verificações anteriores foi útil, pois não haviam peças para serem comidas
        //então o programa verifica as posições recebidas nesse método para permitir um movimento simples da peça para uma outra casa
        if (casaPreta(dx, dy)) { //verifica se a posição de destino é uma casa de cor preta
            if (vez(ox, oy) && casaVazia(dx, dy)) { //verifica a vez do jogador e se a casa destino escolhida está vazia
                if (tabuleiro[ox][oy].getRainha()) { //verifica se a peça é uma dama, para que ela possa se mover para trás
                    if (moverRainha(ox, oy, dx, dy)) { //verifica um movimento válido da dama
                        resultado = 0; //iguala a var. resultado a 0
                    }
                } else { //nesse caso, a peça não é uma dama
                    if (diagonal(ox, oy, dx, dy)) { // como já passou pelas verificações anteriores, só falta verificar se a posição destino está na diagonal da origem
                        resultado = 0; ////iguala a var. resultado a 0
                    }
                }
            } else {
                resultado = 5;
            }

        }
        return resultado; //retorna a variável resultado para casos de apenas mover a peça
    }

    public void verificaDama() {
        //percorre todo o tabuleiro a procura de uma dama
        for (int i = 0; i <= 7; i++) {
            for (int j = 0; j <= 7; j++) {
                if (tabuleiro[i][j] != null) {
                    if (tabuleiro[i][j].getCor()) { //verifica se é uma peça amarela
                        if (i == 0) { //verifica se ele está na primeira linha do tabuleiro, o que indica que ele chegou ao outro lado e é uma dama
                            tabuleiro[i][j].setRainha(true); //altera o atributo da peça para que seja tratada como uma dama
                        }
                    } else { //nao é uma peça amarela mas é uma peça vermelha
                        if (i == 7) { //verifica se a peça está no lado oposto do tabuleiro, o que indica que chegou ao outro lado e é uma dama
                            tabuleiro[i][j].setRainha(true); //altera o atributo da peça para que seja tratada como uma dama
                        }
                    }
                }
            }
        }
    }

    /**
     *  verifica se o destino será uma casa preta e retorna true se sim
     * @param dx destino
     * @param dy destino
     * @return
     */
    private boolean casaPreta(int dx, int dy) {
        if (((dy % 2 == 0) && (dx % 2 == 1)) || ((dy % 2 == 1) && (dx % 2 == 0))) {
            return true;
        }
        return false;
    }

    /**
     * verfica se é a vez do jogador que está exutando a jogada e retorna true se sim
     * @param ox origem
     * @param oy origem
     * @return
     */
    private boolean vez(int ox, int oy) {
        if ((tabuleiro[ox][oy].getCor() == jogadorVez)) {
            return true;
        }
        return false;
    }

    /**
     * verifica se o destino está sem nenhuma peça e retorna true se sim
     * @param dx destino
     * @param dy destino
     * @return
     */
    private boolean casaVazia(int dx, int dy) {
        if ((tabuleiro[dx][dy] == null)) {
            return true;
        }
        return false;
    }

    /**
     * verifica se a jogada está sendo feita na diagonal da origem e returna true se sim
     * @param ox coordenada origem
     * @param oy coordenada origem
     * @param dx coordenada destino
     * @param dy coordenada destino
     * @return
     */
    private boolean diagonal(int ox, int oy, int dx, int dy) {
        if (jogadorVez) {
            if (((dy == oy + 1) || (dy == oy - 1)) && (dx == ox - 1)) {
                return true;
            }
        } else {
            if (((dy == oy + 1) || (dy == oy - 1)) && (dx == ox + 1)) {
                return true;
            }
        }
        return false;
    }

    /**
     * método para que a dama também possa fazer movimentos para trás e retorna true se é possivel
     * @param ox coordenada origem
     * @param oy coordenada origem
     * @param dx coordenada destino
     * @param dy coordenada destino
     * @return
     */
    private boolean moverRainha(int ox, int oy, int dx, int dy) {

        if (((dy == oy + 1) || (dy == oy - 1)) && (dx == ox - 1)) {
            return true;
        }

        if (((dy == oy + 1) || (dy == oy - 1)) && (dx == ox + 1)) {
            return true;
        }

        return false;
    }

    /*
     * recebe como parâmetros as duas posições de origem
     * verifica se a peça comerá a primeira inimiga apenas para frente
     */
    public boolean verificaRota(int ox, int oy) {
        int direcao = 1;
        boolean ok = false;

        if (jogadorVez == true) {
            direcao = -1;
        } else {
            direcao = 1;

        }
        //ESQUERDA
        if (((oy >= 2) && (ox >= 2) && (direcao == -1)) || ((oy >= 2) && (ox <= 5) && (direcao == 1))) {
            if ((tabuleiro[ox + direcao][oy - 1] != null) && (tabuleiro[ox + direcao][oy - 1].getCor() != jogadorVez)) {
                if (tabuleiro[ox + 2 * direcao][oy - 2] == null) {
                    ok = true;
                }
            }
        }

        //DIREITA
        if (((oy <= 5) && (ox >= 2) && (direcao == -1)) || ((oy <= 5) && (ox <= 5) && (direcao == 1))) {
            if ((tabuleiro[ox + direcao][oy + 1] != null) && (tabuleiro[ox + direcao][oy + 1].getCor() != jogadorVez)) {
                if (tabuleiro[ox + 2 * direcao][oy + 2] == null) {
                    ok = true;
                }
            }
        }
        return ok;
    }

    private ArrayList<String> procurarTodosCaminhos() {

        //percorre todo o tabuleiro
        for (int i = 0; i < 8; i++) {
            for (int j = 0; j < 8; j++) {
                if (tabuleiro[i][j] != null) {
                    if (tabuleiro[i][j].getCor() == jogadorVez) {
                        if (this.verificaRota(i, j)) { //garante que a primeira jogada para comer uma peça inimiga, seja feita para frente
                            this.percorreCaminho(i, j, i, j, i + "" + j, ""); //chama o método que verifica se há peças para comer para cada peça encontrada no tabuleiro do jogador da vez
                        } else {
                            if (tabuleiro[i][j].getRainha()) { //caso a peça seja uma dama, a verificação anterior não conta, pois ela pode comer a primeira peça para trás
                                this.percorreCaminho(i, j, i, j, i + "" + j, "");//chama o método que verifica se há peças para comer para cada peça encontrada no tabuleiro do jogador da vez. Válido para a dama também
                            }
                        }
                    }
                }
            }
        }
        // arraylist com todos os caminhos encontrados
        ArrayList<String> copia = caminhos;

        // zerando caminhos
        caminhos = new ArrayList<String>();

        int maior = -1;

        //salva apenas os maiores vetores onde estão as peças para comer
        if (copia.size() > 0 && pecasComer.size() > 0) {
            for (String li : pecasComer) {

                if (li.length() > maior) {
                    maior = li.length();
                    pecasParaComer.clear();
                    pecasParaComer.add(li);
                } else if (li.length() == maior) {
                    pecasParaComer.add(li);
                }
            }
            //arraylist que contém os maiores vetores com as posições de peças para comer
            pecasComer = new ArrayList<String>();

            maior = -1;
            //salva apenas os maiores caminhos encontrados na recursividade
            ArrayList<String> maiorc = new ArrayList<String>();
            for (String linha : copia) {

                if (linha.length() > maior) {
                    maior = linha.length();
                    maiorc.clear();
                    maiorc.add(linha);
                } else if (linha.length() == maior) {
                    maiorc.add(linha);
                }
            }
            //arraylist que contém os maiores caminhos encontrados
            this.maiorCaminho = maiorc;
        }
        return maiorCaminho; //retorna o arraylist de maiores caminhos
    }

    /**
     * Método recursivo para verificar o maior caminho para caputra de peças
     *
     * @param antX linha de onde a peça veio
     * @param antY coluna de onde a peça veio
     * @param xAtual linha atual da peça
     * @param yAtual coluna atual da peça
     * @param anterior todo o caminho por onde a peça passou
     * @param Old todas as peças que serão comidas
     */
    public void percorreCaminho(int antX, int antY, int xAtual, int yAtual, String anterior, String Old) {

        /* verificando direita acima */
        // garantindo que não ocorreu estouro dos limites da matriz e que não volte pela casa de onde veio
        if (((xAtual - 2 >= 0) && (yAtual + 2 <= 7)) && ((xAtual - 2 != antX) || (yAtual + 2 != antY))) {
            // verificando se é possível comer
            if ((tabuleiro[xAtual - 1][yAtual + 1] != null) && (tabuleiro[xAtual - 2][yAtual + 2] == null)) {
                if ((tabuleiro[xAtual - 1][yAtual + 1].getCor() != jogadorVez)) {
                    // verificando se esta jogada não está presente no caminho percorrido até aqui
                    // proteção contra LOOP
                    if (!anterior.contains(xAtual + "" + yAtual + ":" + (xAtual - 2) + "" + (yAtual + 2))) {
                        // adicionando o caminho percorrido no arraylist
                        String novo = anterior + ":" + (xAtual - 2) + "" + (yAtual + 2);
                        caminhos.add(novo);
                        //adicionando a peça que será commida no arraylist
                        String velho = Old + (xAtual - 1) + (yAtual + 1);
                        pecasComer.add(velho);

                        // será que deste ponto é possível capturar outra peça? vamos ver
                        this.percorreCaminho(xAtual, yAtual, xAtual - 2, yAtual + 2, novo, velho);
                    } else {
                        // mensagem de depuração
                        System.err.println("teste  : " + xAtual + "" + yAtual + ":" + (xAtual - 2) + "" + (yAtual + 2));
                        System.err.println("caminho: " + anterior);
                    }
                }
            }
        }

        /* verificando direita abaixo */
        // garantindo que não ocorreu estouro dos limites da matriz e que não volte pela casa de onde veio
        if (((xAtual + 2 <= 7) && (yAtual + 2 <= 7)) && ((xAtual + 2 != antX) || (yAtual + 2 != antY))) {
            // verificando se é possível comer
            if ((tabuleiro[xAtual + 1][yAtual + 1] != null) && (tabuleiro[xAtual + 2][yAtual + 2] == null)) {
                if ((tabuleiro[xAtual + 1][yAtual + 1].getCor() != jogadorVez)) {
                    // verificando se esta jogada não está presente no caminho percorrido até aqui
                    // proteção contra LOOP
                    if (!anterior.contains(xAtual + "" + yAtual + ":" + (xAtual + 2) + "" + (yAtual + 2))) {
                        // adicionando o caminho percorrido no arraylist
                        String novo = anterior + ":" + (xAtual + 2) + "" + (yAtual + 2);
                        caminhos.add(novo);
                        //adicionando a peça que será commida no arraylist
                        String velho = Old + (xAtual + 1) + (yAtual + 1);
                        pecasComer.add(velho);
                        // será que deste ponto é possível capturar outra peça? vamos ver
                        this.percorreCaminho(xAtual, yAtual, xAtual + 2, yAtual + 2, novo, velho);
                    } else {
                        // mensagem de depuração
                        System.err.println("teste  : " + xAtual + "" + yAtual + ":" + (xAtual + 2) + "" + (yAtual + 2));
                        System.err.println("caminho: " + anterior);
                    }
                }
            }
        }


        // esquerda acima
        // garantindo que não ocorreu estouro dos limites da matriz e que não volte pela casa de onde veio
        if (((xAtual - 2 >= 0) && (yAtual - 2 >= 0)) && ((xAtual - 2 != antX) || (yAtual - 2 != antY))) {
            // verificando se é possível comer
            if ((tabuleiro[xAtual - 1][yAtual - 1] != null) && (tabuleiro[xAtual - 2][yAtual - 2] == null)) {
                if ((tabuleiro[xAtual - 1][yAtual - 1].getCor() != jogadorVez)) {
                    // verificando se esta jogada não está presente no caminho percorrido até aqui
                    // proteção contra LOOP
                    if (!anterior.contains(xAtual + "" + yAtual + ":" + (xAtual - 2) + "" + (yAtual - 2))) {
                        // adicionando o caminho percorrido no arraylist
                        String novo = anterior + ":" + (xAtual - 2) + "" + (yAtual - 2);
                        caminhos.add(novo);
                        //adicionando a peça que será commida no arraylist
                        String velho = Old + (xAtual - 1) + (yAtual - 1);
                        pecasComer.add(velho);
                        // será que deste ponto é possível capturar outra peça? vamos ver
                        this.percorreCaminho(xAtual, yAtual, xAtual - 2, yAtual - 2, novo, velho);
                    } else {
                        //mensagem de depuração
                        System.err.println("teste  : " + xAtual + "" + yAtual + ":" + (xAtual - 2) + "" + (yAtual - 2));
                        System.err.println("caminho: " + anterior);
                    }

                }
            }
        }



        // esquerda abaixo
        // garantindo que não ocorreu estouro dos limites da matriz e que não volte pela casa de onde veio
        if (((xAtual + 2 <= 7) && (yAtual - 2 >= 0)) && ((xAtual + 2 != antX) || (yAtual - 2 != antY))) {
            // verificando se é possível comer
            if ((tabuleiro[xAtual + 1][yAtual - 1] != null) && (tabuleiro[xAtual + 2][yAtual - 2] == null)) {
                if ((tabuleiro[xAtual + 1][yAtual - 1].getCor() != jogadorVez)) {
                    // verificando se esta jogada não está presente no caminho percorrido até aqui
                    // proteção contra LOOP
                    if (!anterior.contains(xAtual + "" + yAtual + ":" + (xAtual + 2) + "" + (yAtual - 2))) {
                        // adicionando o caminho percorrido no arraylist
                        String novo = anterior + ":" + (xAtual + 2) + "" + (yAtual - 2);
                        caminhos.add(novo);
                        //adicionando a peça que será commida no arraylist
                        String velho = Old + (xAtual + 1) + (yAtual - 1);
                        pecasComer.add(velho);
                        // será que deste ponto é possível capturar outra peça? vamos ver
                        this.percorreCaminho(xAtual, yAtual, xAtual + 2, yAtual - 2, novo, velho);
                    } else {
                        //mensagem de depuração
                        System.err.println("teste  : " + xAtual + "" + yAtual + ":" + (xAtual + 2) + "" + (yAtual - 2));
                        System.err.println("caminho: " + anterior);
                    }
                }
            }
        }
    }
}
